#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""

from forest import *
from tree import *
from branch import *
from domain import *
from instance import *


class DataReader:
    """
    Class used to parse the input files.
    """

    def __init__(self):
        """
        Constructor.
        """
        pass

    def parseBoostedTree(self, inputFile):
        """
        Read from a given input file a forest.

        Attribute:
        inputFile: str
            is the input file.

        :returns: a forest.
        :rtype: Forest
        """
        ret = Forest()

        with open(inputFile) as f:
            for line in f:
                tree = Tree()

                # create the tree regarding the parsed line.
                for b in line.split(';'):
                    data = b.split()
                    if(len(data) == 0): 
                        continue                    
                    literals = [int(l) for l in data[:-1]]
                    tree.addBranch(Branch(literals, float(data[-1])))

                # add the tree to the forest.
                ret.addTree(tree)

        return ret

    def parseTheory(self, inputFile):
        """
        Read the theory from a file. The input format for the theory is the DIMACS format.

        Args:
        inputFile: str
            is the input file

        :returns: a set of clauses.
        :rtype: list(list(int))
        """
        theory = Domain()

        with open(inputFile) as f:
            for line in f:
                p = line.split()

                if(len(p) <= 3 or p[2] != '>='): # categorical
                    for b in p[1:]:
                        theory.addCatVar(p[0], int(b))
                else:
                    assert len(p) == 4
                    theory.addNumVar(p[0], int(p[1]), float(p[3]))

        return theory

    def parseInstances(self, inputFile) -> list:
        """
        Read from a file the set of instances we want to explain.

        Args:
        inputFile: str
            is the input file

        :returns: a set instances.
        :rtype: list(list(int))
        """        
        instances = []
        
        with open(inputFile) as f:
            literals = []
            prediction = 0

            lines = f.readlines()
            i = 0
            while i<len(lines) :
                lines[i].lstrip()
                if len(lines[i]) == 0 or lines[i][0] == '\n':  # we suppose that an empty also mean we reach the end of the file.                    
                    break
            
                literals = []
                for l in lines[i].split(',')[:-1]:
                    literals.append(int(l))
                prediction = float(lines[i].split(',')[-1])                
                
                i += 1                
                
                scoringMap = {}
                while lines[i][0] != '%':
                    cut = lines[i].split()
                    if len(cut) == 2:
                        assert cut[0] not in scoringMap
                        if cut[1] == 'inf':
                            scoringMap[cut[0]] = ['inf']
                        else:
                            scoringMap[cut[0]] = [float(cut[1])]
                    else:                    
                        if cut[0] not in scoringMap:
                            scoringMap[cut[0]] = []                                                    
                        scoringMap[cut[0]].append(cut[1:])
                    i += 1                    
                i += 1
                    
                
                instances.append(Instance(literals, prediction, scoringMap))                
            
        return instances
