#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""

from variable import *

class Domain:
    """
    Class to collect information about the domains.
    """

    def __init__(self) -> None:
        """
        Constructor.
        """
        self.mapping = {}
        pass


    def addNumVar(self, var, boolVar, value):
        """
        Add a numeric variable.
        Attributs:
        ---------
            var : str
                gives the variable name as in the data set.
            boolVar : int
                gives the id of the propositional variable associate with the value.
            value : int
                we have boolVar <=> var >= value
        """
        
        if var not in self.mapping:
            self.mapping[var] = NumericalVar(var)
        assert self.mapping[var].getType() == VariableType.NUMERICAL
        self.mapping[var].addThreshold(boolVar, value)
               

    def addCatVar(self, var, boolVar):
        """
        Add a categorical variable.
        Attributs:
        ---------
            var : str
                gives the variable name as in the data set.
            boolVar : int
                gives the id of the propositional variable associate the category in the BT.
        """
        if var not in self.mapping:
            self.mapping[var] = CategoricalcalVar(var)
        assert self.mapping[var].getType() == VariableType.CATEGORICAL
        self.mapping[var].addValue(boolVar)
        

    def generateTheoryClauses(self):
        """
        Generate the constraint enforce by the domain theory as a set of clauses.

        :returns: a list of clauses.
        :rtype: List(List(int))
        """
        clauses = []
        for k in self.mapping:
            clauses += self.mapping[k].generateDomainClauses()

        return clauses