# usage python main.py dataset.csv dataset.types  outputdir
import sys
import os
import re
import random
def extract_branches(node, branches, weights, current):
    if isinstance(node, float) or isinstance(node, int):
        weights.append(node)
        branches.append(current.copy())
        return
    assert (len(node) == 3)

    # left (false branch)
    current.append(-node[0])
    extract_branches(node[1], branches, weights, current)
    current.pop()

    # right (true branch
    current.append(node[0])
    extract_branches(node[2], branches, weights, current)
    current.pop()



if len(sys.argv) != 3:
    print("usage: python main.py dataset outputdir")
    sys.exit(1)

from pyxai import Learning, Explainer

dir = sys.argv[2]
dataset = os.path.basename(sys.argv[1]+ ".csv")
learner = Learning.Xgboost(sys.argv[1]+ ".csv", learner_type=Learning.REGRESSION)
models = learner.evaluate(method=Learning.K_FOLDS, output=Learning.BT, n_models=10)

for i, model in enumerate(models):
    explainer = Explainer.initialize(model, features_type=sys.argv[1] + ".types")

    # export branches
    f = open(f"{dir}/{dataset}-trees-{i}.txt", "w")
    for tree in explainer.regression_boosted_trees.forest:
        branches = []
        current = []
        weights = []
        all_branches = []
        raw = tree.raw_data_for_CPP()
        raw = raw[1]
        if len(raw) == 1:  # just one node
            branches.append([])
            weights.append(raw[0])
        else:
            extract_branches(raw, branches, weights, current)
        assert (len(branches) == len(weights))
        for j in range(len(branches)):
            b = " ".join(map(str,branches[j]))+" "+ str(weights[j])
            all_branches.append(b)
        f.write(";".join(all_branches)+"\n")
    f.close()



    f = open(f"{dir}/{dataset}-theory-{i}.txt", "w")
    # export theory
    categorical = {}
    numerical = []
    numerical_features = set()
    categorical_features = set()
    instance, prediction = learner.get_instances(model, n=1)
    explainer.set_instance(instance)
    for j in range(1, len(explainer.binary_representation)+1):
        d = explainer.to_features([j], eliminate_redundant_features=True, details=True)
        key = (list(d.keys())[0])
        nkey = re.sub(' ', '', key)
        #t = d[key][""]
        #print(j)
        #print(d[key][0])
        if d[key][0]["theory"] == "numerical":
            tmp = str(j) + " >= " + str(d[key][0]["threshold"])
            numerical.append(nkey + " " + tmp)
            numerical_features.add(nkey)
        else:
            if key in categorical:
                categorical[nkey] = categorical[key] + " " + str(j)
            else:
                categorical[nkey] = str(j)
                categorical_features.add(nkey)
        #print(s)
        #tmp.append()
    numerical = sorted(numerical)
    for (k,v) in categorical.items():
        f.write(k + " " + v + "\n")
    for n in numerical:
        f.write(n + "\n")
    #print(instance)
    #print(explainer.binary_representation)
    f.close()
    print(numerical_features)
    print(categorical_features)
    names = model.learner_information.feature_names

    instances = learner.get_instances(model, n=10)
    f = open(f"{dir}/{dataset}-instances-{i}.txt", "w")
    # extract instances
    instances = learner.get_instances(model, n=10)
    # f = open(f"{dir}/{file}-instances-{i}.txt")
    for instance, prediction in instances:
        explainer.set_instance(instance)
        done = []
        tmp = ','.join(map(str,explainer.binary_representation)) + "," + str(prediction)
        f.write(tmp+"\n")
        for i, lit in enumerate(instance):
            nkey =  re.sub(' ', '', names[i])
            st = random.randint(1,100)
            if st <= 20:
                if nkey in numerical_features:
                    f.write(nkey + " inf\n")
                else:
                    cat = nkey[0:nkey.find("_")]
                    if cat in categorical_features:
                        if cat not in done:
                            done.append(cat)
                            f.write(re.sub(' ', '', cat) + " inf\n")
            else:
                if nkey in numerical_features:
                    f.write(nkey + " 0 " + str(lit*0.9) + " "+  str(lit *1.1) + "\n")
                    f.write(nkey + " 1 " + str(lit*0.75) + " "+str(lit*0.9) +  " "+str(lit*1.1) +" "+ str(lit*1.25)+ "\n")
                    f.write(nkey + " 5 "  + str(lit *0.5) + " "+str(lit *0.75) + " "+ str(lit*1.25) + " "+str(lit*1.50)+ "\n")
                    f.write(nkey + " inf " + str(lit*0.5) + " "+str(lit*1.50)+ "\n")

                else:
                    cat = nkey[0:nkey.find("_")]
                    if cat in categorical_features:
                        if cat not in done:
                            done.append(cat)
                            f.write(re.sub(' ', '', cat) + " 1\n")

        f.write("%\n") #end
    f.close()
