#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""

from tree import Tree
from branch import Branch


class Forest:
    """
    Class to store data about a given forest.
    """

    def __init__(self) -> None:
        """
        Construct a problem.

        Attribute:
        ----------
        trees : list(Tree)
            gives the list of branches.
        """
        self.trees = []
        self.maxIndex = 0
        self.nbBranchAllTree = 0

    def addTree(self, tree: Tree) -> None:
        """
        Add a tree to the forest.

        Attribute:
        ----------
        tree : Tree
            is the tree we want to add.
        """
        self.trees.append(tree)

        for b in tree:
            for l in b.literals:
                if (abs(l) > self.maxIndex):
                    self.maxIndex = abs(l)
        self.nbBranchAllTree += tree.getNbBranch()

    def __iter__(self):
        ''' Returns the Iterator object '''
        return iter(self.trees)

    def getMaxIndex(self):
        """        
        :returns: the maximum index.
        :rtype: int
        """
        return self.maxIndex

    def getNbAllBranches(self):
        """        
        :returns: the number of branches when considering all trees.
        :rtype: int
        """
        return self.nbBranchAllTree

    def getMaxPossiblePrediction(self):
        """
        :returns: sum all the maximum value for the trees.
        :rtype: float
        """
        ret = 0
        for t in self.trees:
            ret += t.getMaxValue()
        return ret

    def getMinPossiblePrediction(self):
        """
        :returns: sum all the maximum value for the trees.
        :rtype: float
        """
        ret = 0
        for t in self.trees:
            ret += t.getMinValue()
        return ret

    def getNbTree(self):
        """        
        :returns: the number of tree.
        :rtype: int
        """
        return len(self.trees)

    def display(self):
        """
        Display information about the problem.
        """
        cpt = 0
        for t in self.trees:
            cpt += 1
            print("[Tree]", cpt)
            t.display()

    def computePrediction(self, interpretation):
        """        
        :returns: the prediction.
        :rtype: float
        """
        ret = 0

        # rewrite the interpretation to align with int
        interpretationAligned = []
        for l in interpretation:
            while abs(l) >= len(interpretationAligned):
                interpretationAligned.append(0)
            interpretationAligned[abs(l)] = l

        for t in self.trees:
            ret += t.computePrediction(interpretationAligned)
        return ret
