#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""


class Instance:
    """
    Class used to store information about the instance we want to explain.    
    """
    
    def __init__(self, literals, prediction, scoringMap) -> None:
        """
        Constructor.        
        
        Attributs:
        ---------
            literals : List(int)
                gives the list of boolean features of the instance.
            prediction : float
                gives the prediction associate with the literals w.r.t. the BT
            scoringMap : Map
                gives for each feature the scores assigned when changing.
        """
        self.literals = literals
        self.prediction = prediction
        self.scoringMap = scoringMap
        
    def getWeightCategorical(self, var, value):
        """
        Get the weight associate with a categorical variable given a value.

        Args:
            var (int): the variable we want to know the weight.
            value (int): the value.
            
        :returns: the weight given in the scoring map for the variable var.
        :rtype: float
        """
        if var not in self.scoringMap:
            return 0
        
        if len(self.scoringMap[var]) == 1:            
            return self.scoringMap[var][0]
        
        for c in self.scoringMap[var]:
            if int(c[0]) == value:
                if c[1] == 'inf':
                    return 'inf'
                return float(c[1])
        return 0
    
    def getWeightNumerical(self, var, l, r):
        """
        Here we suppose the variable under consideration represents a numerical feature.

        Args:
            var (str): the variable name
            l (float): the left part of the interval.
            r (float): the right part of the interval.
        """      
        if var not in self.scoringMap:
            return 0
        
        if len(self.scoringMap[var]) == 1:
            return 'inf'
        
        if type(l) == str:
            l = -1.7976931348623157e+308
        if type(r) == str:
            r = 1.7976931348623157e+308
        
        min = None
        for c in self.scoringMap[var]:
            for i in range(1, len(c), 2):             
                if l >= float(c[i+1]):
                    continue
                if r < float(c[i]):
                    continue
                
                if min == None or (min == 'inf' and c[0] != 'inf') or (min != 'inf' and min > float(c[0])):
                    if c[0] != 'inf':
                        min = float(c[0])
                    else:
                        min = 'inf'
        
        if min == None:
            return 0
        return min
                
        
    def __str__(self) -> str:
        return "X = " + str(self.literals) + "\nprediction = " + str(self.prediction) + "\nScoring Map = " + str(self.scoringMap)
