#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""


import argparse
from dataReader import *
from contrastive import *
import gurobipy as gp

import sys

if __name__ == '__main__':
    print("BTEX")

    parser = argparse.ArgumentParser(description='Process arguments.')
    parser.add_argument('-f', type=str, dest='forestFile', nargs='?', default='/dev/stdin',
                        help='input file containing the boosted tree.')
    parser.add_argument('-t', type=str, dest='theoryFile', nargs='?', default='/dev/stdin',
                        help='input file containing the theory (i.e. a set of clauses).')
    parser.add_argument('-i', type=str, dest='instanceFile', nargs='?', default='/dev/stdin',
                        help='input file containing the set of instances we want to explain.')
    args = parser.parse_args()

    parser = DataReader()
    bt = parser.parseBoostedTree(args.forestFile)
    
    domains = parser.parseTheory(args.theoryFile)    

    # computeInterval = Contrastive(timeLimit=100.0)
    # bound = computeInterval.runMinMaxInterval(bt, domains)
    # print('s', bound[0], bound[3])
    
    # sys.exit(0)


    instances = parser.parseInstances(args.instanceFile)
        
    results = []
    
    minVal = bt.getMaxPossiblePrediction()
    maxVal = bt.getMinPossiblePrediction()
    for instance in instances:
        if instance.prediction < minVal:
            minVal = instance.prediction
        if instance.prediction > maxVal:
            maxVal = instance.prediction
        
    print("c minVal:", minVal)
    print("c manVal:", maxVal)
    block = abs(maxVal - minVal) / 100
    
    for instance in instances:        
        for shift in [-5, -10, -20, -50, 5, 10, 20, 50]:
            for sInt in [1, 2, 3]:
                sizeInterval = sInt * block * 0.5
                xai = Contrastive(timeLimit=100.0)
                            
                lb = round(instance.prediction + shift * block - sizeInterval, 2)
                ub = round(instance.prediction + shift * block + sizeInterval, 2)

                t = xai.run(bt, domains, instance, lb, ub)
                print('s', t[0], t[1], shift, sInt, t[2], lb, ub, instance.prediction)
                results.append(t + tuple([shift, sInt]))

