#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""


class Tree:
    """
    Class to store data about a given tree.
    """
    nbTree = 0

    def __init__(self):
        """
        Construct a problem.
        Attributes:
        ----------
        branches : list(Branch)
            gives the list of branches.
        """
        self.branches = []
        self.minValue = 1.7976931348623157e+308
        self.maxValue = -1.7976931348623157e+308
        self.id = Tree.nbTree
        Tree.nbTree += 1

    def addBranch(self, branch):
        """
        Add a tree to the forest.

        Attribute:
        ----------
        branch : Branch
            is the branch we want to add.
        """
        self.branches.append(branch)
        if (self.minValue > branch.getValue()):
            self.minValue = branch.getValue()

        if (self.maxValue < branch.getValue()):
            self.maxValue = branch.getValue()

    def getNbBranch(self):
        """        
        :returns: the number of branches.
        :rtype: int
        """
        return len(self.branches)

    def getMinValue(self):
        """        
        :returns: the minimum weight in the tree.
        :rtype: float
        """
        return self.minValue

    def getId(self):
        """        
        :returns: the id of the tree.
        :rtype: float
        """
        return self.id

    def getMaxValue(self):
        """        
        :returns: the maximum weight in the tree.
        :rtype: float
        """
        return self.maxValue

    def __iter__(self):
        ''' Returns the Iterator object '''
        return iter(self.branches)

    def display(self):
        """
        Display information about the problem.
        """
        for b in self.branches:
            b.display()

    def computePrediction(self, interpretation):
        """        
        :returns: the prediction.
        :rtype: float
        """
        for b in self.branches:
            pathOk = True
            for l in b.getLiterals():
                if interpretation[abs(l)] != l:
                    pathOk = False
                    break
            if pathOk:
                return b.getValue()

        print("interpretation = ", interpretation)
        self.display()
        assert False
