#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: JM
"""

from enum import Enum

class VariableType(Enum):
    CATEGORICAL = 1
    NUMERICAL = 2    

class Variable:
    """
    Class to represent a variable.
    """

    def __init__(self, name : str) -> None:
        """
        Constructor.
        """
        self.name = name    


class NumericalVar(Variable):
    """
    Class to represent a numerical variable.
    """

    def __init__(self, name: str) -> None:
        """
        Constructor.

        Attributs:
            name : str
                gives the name of the variable.
        """
        super().__init__(name)
        self.threshold = []


    def getType(self) -> VariableType:
        """
        Give the type of the variable.

        :returns: the type.
        :rtype: VariableType
        """
        return VariableType.NUMERICAL


    def addThreshold(self, boolVar : int, value : int):
        """
        Add a new threshold, such that boolVar <=> self.name >= value.

        Attributs:
            boolVar : int
                gives the boolean variable index.
            value : int
                gives the threshold.
        """
        self.threshold.append((boolVar, value))

    def generateDomainClauses(self):
        """
        Generate the clause that encodes the domain theory.

        :returns: a list of clauses.
        :rtype: List(List(int))
        """
        clauses = []

        self.threshold = sorted(self.threshold, key = lambda x : x[1])
        for i in range(len(self.threshold) - 1, 0, -1):            
            clauses.append([-self.threshold[i][0], self.threshold[i-1][0]])

        return clauses
        

class CategoricalcalVar(Variable):
    """
    Class to represent a categorical variable.
    """

    def __init__(self, name: str) -> None:
        """
        Constructor.

        Attributs:
            name : str
                gives the name of the variable.
        """
        super().__init__(name)
        self.values = []


    def getType(self) -> VariableType:
        """
        Give the type of the variable.

        :returns: the type.
        :rtype: VariableType
        """
        return VariableType.CATEGORICAL


    def addValue(self, boolVar : int):
        """
        Add a new value to the domain.

        Attributs:
            boolVar : int
                gives the boolean variable index.            
        """
        self.values.append(boolVar)

    def generateDomainClauses(self):
        """
        Generate the clause that encodes the domain theory.

        :returns: a list of clauses.
        :rtype: List(List(int))
        """
        clauses = []
        
        for i in range(len(self.values)):
            for j in range(i+1, len(self.values)):                
                clauses.append([-self.values[i], -self.values[j]])

        return clauses